@@ -101,7 +101,7 @@ class ElasticSearchDialect(DialectBase):
         return qr
 
     @staticmethod
-    def create_term(path, value, multiple=False, all_resources=False):
+    def create_term(path, value, multiple=False, match_type=None, all_resources=False):
         """Create ES Query term"""
         multiple_ = isinstance(value, (list, tuple)) or multiple is True
         if multiple_ is True and not isinstance(value, (list, tuple)):
@@ -111,6 +111,8 @@ class ElasticSearchDialect(DialectBase):
             # TODO is there a better way to search on all resource types?
             # TODO does it work if both all_resources and multiple_ are True?
             q = {"multi_match": {"query": value, "fields": [path]}}
+        elif match_type == TermMatchType.EXACT:
+            q = {"term": {f"{path}.raw": value}}
         elif multiple_:
             q = {"terms": {path: value}}
         else:
@@ -367,8 +369,11 @@ class ElasticSearchDialect(DialectBase):
                 if term.type == GroupType.COUPLED:
                     alsoProvides(t_, IIgnoreNestedCheck)
 
-                resolved = self.resolve_term(t_, mapping, root_replacer)
-                container.append(resolved[0])
+                resolved, operator = self.resolve_term(t_, mapping, root_replacer)
+                if operator == OPERATOR.neg:
+                    container.append({"bool": {"must_not": [resolved]}})
+                else:
+                    container.append(resolved)
 
             if not IIgnoreNestedCheck.providedBy(term):
                 qr = ElasticSearchDialect.attach_nested_on_demand(
@@ -447,6 +452,7 @@ class ElasticSearchDialect(DialectBase):
                                 dotted_path,
                                 value,
                                 multiple=multiple,
+                                match_type=term.match_type,
                                 all_resources=all_resources,
                             )
                         resolved = q, term.unary_operator
@@ -490,27 +496,22 @@ class ElasticSearchDialect(DialectBase):
             return self.resolve_nonfhir_term(term)
 
     def resolve_datetime_term(self, term, root_replacer=None):
-        """TODO: 1.) Value Conversion(stringify) based of context.type_name
-        i.e date or dateTime or Time"""
+        """ """
         qr = dict()
-        if INonFhirTerm.providedBy(term):
-            type_name = term.value.__visit_name__
-        else:
-            type_name = term.path.context.type_name
 
         value = term.get_real_value()
         path_ = ElasticSearchDialect.create_dotted_path(term, root_replacer)
 
-        if type_name in ("dateTime", "instant"):
+        if hasattr(value, "day") and hasattr(value, "hour"):
             value_formatter = (
                 isodate.DATE_EXT_COMPLETE + "T" + isodate.TIME_EXT_COMPLETE
             )
-        elif type_name == "date":
+        elif hasattr(value, "day"):
             value_formatter = isodate.DATE_EXT_COMPLETE
-        elif type_name == "time":
+        elif hasattr(value, "hour"):
             value_formatter = isodate.TIME_EXT_COMPLETE
         else:
-            raise NotImplementedError
+            raise ValueError("Could not understand date query value.")
 
         if term.comparison_operator in (OPERATOR.eq, OPERATOR.ne):
             qr["range"] = {
@@ -538,7 +539,7 @@ class ElasticSearchDialect(DialectBase):
                 }
             }
 
-        if type_name in ("dateTime", "instant", "time") and value.tzinfo:
+        if hasattr(value, "tzinfo") and value.tzinfo:
             timezone = isodate.tz_isoformat(value)
             if timezone not in ("", "Z"):
                 qr["range"][path_]["time_zone"] = timezone
