@@ -2,56 +2,60 @@
 import ast
 import datetime
 import re
+from abc import ABC
 from collections import deque
 from copy import copy
 
 import isodate
-from zope.interface import implementer
-from zope.interface import implementer_only
+from zope.interface import implementer, implementer_only
 
-from fhirpath.constraints import required_finalized
-from fhirpath.constraints import required_not_finalized
-from fhirpath.constraints import required_value_not_assigned
-from fhirpath.enums import OPERATOR
-from fhirpath.enums import GroupType
-from fhirpath.enums import MatchType
-from fhirpath.enums import SortOrderType
-from fhirpath.enums import TermMatchType
-from fhirpath.enums import WhereConstraintType
+from fhirpath.constraints import (
+    required_finalized,
+    required_not_finalized,
+    required_value_not_assigned,
+)
+from fhirpath.enums import (
+    OPERATOR,
+    GroupType,
+    MatchType,
+    SortOrderType,
+    TermMatchType,
+    WhereConstraintType,
+)
 from fhirpath.exceptions import ValidationError
 from fhirpath.interfaces import IElementPath
 from fhirpath.interfaces.base import IFhirPrimitiveType
-from fhirpath.interfaces.fql import IBaseTerm
-from fhirpath.interfaces.fql import IExistsGroupTerm
-from fhirpath.interfaces.fql import IExistsTerm
-from fhirpath.interfaces.fql import IFqlClause
-from fhirpath.interfaces.fql import IGroupTerm
-from fhirpath.interfaces.fql import IInTerm
-from fhirpath.interfaces.fql import INonFhirTerm
-from fhirpath.interfaces.fql import IPathConstraint
-from fhirpath.interfaces.fql import ISortTerm
-from fhirpath.interfaces.fql import ITerm
-from fhirpath.interfaces.fql import ITermValue
-from fhirpath.interfaces.fql import IValuedClass
-from fhirpath.types import EMPTY_VALUE
-from fhirpath.types import FhirBoolean
-from fhirpath.types import FhirDate
-from fhirpath.types import FhirDateTime
-from fhirpath.types import FhirDecimal
-from fhirpath.types import FhirInteger
-from fhirpath.types import FhirString
-from fhirpath.utils import EmptyPathInfoContext
-from fhirpath.utils import PathInfoContext
-from fhirpath.utils import proxy
-from fhirpath.utils import unwrap_proxy
-
+from fhirpath.interfaces.fql import (
+    IBaseTerm,
+    IExistsGroupTerm,
+    IExistsTerm,
+    IFqlClause,
+    IGroupTerm,
+    IInTerm,
+    INonFhirTerm,
+    IPathConstraint,
+    ISortTerm,
+    ITerm,
+    ITermValue,
+    IValuedClass,
+)
+from fhirpath.types import (
+    EMPTY_VALUE,
+    FhirBoolean,
+    FhirDate,
+    FhirDateTime,
+    FhirDecimal,
+    FhirInteger,
+    FhirString,
+)
+from fhirpath.utils import EmptyPathInfoContext, PathInfoContext, proxy, unwrap_proxy
 
 __author__ = "Md Nazrul Islam<email2nazrul@gmail.com>"
 
 has_dot_as = re.compile(r"\.as\((?P<type_name>[a-z]+)\)$", re.I | re.U)
-has_space_as = re.compile(r"^[a-z\.0-9]+ +as +[a-z0-9]+$", re.I | re.U)
+has_space_as = re.compile(r"^[a-z.0-9]+ +as +[a-z0-9]+$", re.I | re.U)
 has_dot_is = re.compile(r"\.is\([a-z]+\)$", re.I | re.U)
-has_dot_where = re.compile(r"\.where\([a-z\=\'\"\(\)\s\-]+\)", re.I | re.U)
+has_dot_where = re.compile(r"\.where\([a-z=\'\"()\s\-]+\)", re.I | re.U)
 
 contains_index = re.compile(r"\[[0-9]+\]", re.U)
 # first()last()Tail()count()Skip(1).Take(3)
@@ -92,7 +96,7 @@ class SortClause(FqlClause):
 
 
 @implementer(IFqlClause)
-class LimitClause(object):
+class LimitClause(ABC):
     """ """
 
     __slots__ = ("_limit", "_offset")
@@ -129,7 +133,7 @@ class LimitClause(object):
 
 
 @implementer(IBaseTerm, IValuedClass)
-class BaseTerm(object):
+class BaseTerm(ABC):
     """ """
 
     def __init__(self, path, value=EMPTY_VALUE, match_type=None):
@@ -142,7 +146,7 @@ class BaseTerm(object):
 
         # eq, ne, lt, le, gt, ge
         self.comparison_operator = None
-        # +,- (negetive, positive)
+        # +,- (negative, positive)
         self.unary_operator = None
         # and, or, xor
         self.arithmetic_operator = None
@@ -182,7 +186,9 @@ class BaseTerm(object):
 
     def set_match_type(self, type_):
         """ """
-        self.match_type = TermMatchType[type_]
+        if isinstance(type_, str):
+            type_ = TermMatchType[type_]
+        self.match_type: TermMatchType = type_
 
     def clone(self):
         """ """
@@ -345,8 +351,8 @@ class Term(BaseTerm):
     def validate(self):
         """ """
         # xxx: required validate ```comparison_operator```
-        # lt,le,gt,ge only for Date,DateTime, Interger, Float
-        if IFhirPrimitiveType.implementedBy(self.path.context.type_class):
+        # lt,le,gt,ge only for Date,DateTime, Integer, Float
+        if self.path.context.type_is_primitive:
             if self.path.context.type_name not in (
                 "integer",
                 "decimal",
@@ -363,9 +369,8 @@ class Term(BaseTerm):
                 OPERATOR.ge,
             ):
                 raise ValidationError(
-                    "Operator '{0!s}' is allowed for value type '{1!s}'".format(
-                        self.comparison_operator.__name__, self.path.context.type_name
-                    )
+                    f"Operator '{self.comparison_operator}' is not allowed "
+                    f"for value type {self.path.context.type_name}'"
                 )
         else:
             # don't have usecase yet!
@@ -455,7 +460,7 @@ class NonFhirTerm(BaseTerm):
     def validate(self):
         """ """
         # xxx: required validate ```comparison_operator```
-        # lt,le,gt,ge only for Date,DateTime, Interger, Float
+        # lt,le,gt,ge only for Date,DateTime, Integer, Float
         if self.value.__visit_name__ not in (
             "integer",
             "decimal",
@@ -563,7 +568,7 @@ class ExistsTerm(object):
 
         # Path Context
         self.context = None
-        # +,- (negetive, positive)
+        # +,- (negative, positive)
         self.unary_operator = None
 
         if isinstance(path, str):
@@ -670,14 +675,7 @@ class TermValue(object):
         """context: PathInfoContext """
         required_not_finalized(self)
         path = IElementPath(path)
-        value = path.context.type_class(self.raw)
-
-        if IFhirPrimitiveType.providedBy(value):
-            self.value = value.to_python()
-        else:
-            # xxx: support for other value type
-            raise NotImplementedError
-
+        self.value = path.context.validate_value(self.raw)
         self._finalized = True
 
     def __call__(self):
@@ -745,12 +743,12 @@ class GroupTerm(object):
         for term in self.terms:
             term.finalize(context)
 
-        if self.match_operator is None:
-            self.match_operator = MatchType.ANY
-
         if self.type is None:
             self.type = GroupType.COUPLED
 
+        if self.match_operator is None:
+            self.match_operator = MatchType.ANY
+
         self._finalized = True
 
     def __copy__(self):
@@ -782,6 +780,11 @@ class GroupTerm(object):
         self.match_operator = MatchType.ANY
         return self.clone()
 
+    def match_no_one(self):
+        """ """
+        self.match_operator = MatchType.NONE
+        return self.clone()
+
 
 @implementer(IExistsGroupTerm)
 class ExistsGroupTerm(object):
@@ -860,6 +863,11 @@ class ExistsGroupTerm(object):
         self.match_operator = MatchType.ANY
         return self.clone()
 
+    def match_no_one(self):
+        """ """
+        self.match_operator = MatchType.NONE
+        return self.clone()
+
 
 @implementer(ISortTerm)
 class SortTerm(object):
