@@ -1,45 +1,46 @@
 # _*_ coding: utf-8 _*_
+from abc import ABC
 from copy import copy
-from typing import Optional
+from typing import TYPE_CHECKING, List, Optional, Union
 
 from zope.interface import implementer
 
 from fhirpath.enums import EngineQueryType
-from fhirpath.exceptions import ConstraintNotSatisfied
-from fhirpath.exceptions import ValidationError
+from fhirpath.exceptions import ConstraintNotSatisfied, ValidationError
 from fhirpath.model import Model
 from fhirpath.thirdparty import Proxy
-from fhirpath.utils import FHIR_VERSION
-from fhirpath.utils import builder
+from fhirpath.utils import FHIR_VERSION, builder
 
-from .constraints import required_finalized
-from .constraints import required_from_resource
-from .constraints import required_not_finalized
+from .constraints import required_finalized, required_not_finalized
 from .exceptions import MultipleResultsFound
-from .fql.expressions import and_
-from .fql.expressions import fql
-from .fql.expressions import sort_
-from .fql.types import ElementPath
-from .fql.types import FromClause
-from .fql.types import LimitClause
-from .fql.types import SelectClause
-from .fql.types import SortClause
-from .fql.types import WhereClause
-from .interfaces import ICloneable
-from .interfaces import IElementPath
-from .interfaces import IGroupTerm
-from .interfaces import IQuery
-from .interfaces import IQueryBuilder
-from .interfaces import IQueryResult
-from .interfaces import ISortTerm
-from .interfaces import ITerm
+from .fql.expressions import and_, fql, sort_
+from .fql.types import (
+    ElementPath,
+    FromClause,
+    LimitClause,
+    SelectClause,
+    SortClause,
+    WhereClause,
+)
+from .interfaces import (
+    ICloneable,
+    IElementPath,
+    IGroupTerm,
+    IQuery,
+    IQueryBuilder,
+    IQueryResult,
+    ISortTerm,
+    ITerm,
+)
 
+if TYPE_CHECKING:
+    from fhirpath.engine.base import Engine
 
 __author__ = "Md Nazrul Islam<email2nazrul@gmail.com>"
 
 
 @implementer(IQuery, ICloneable)
-class Query(object):
+class Query(ABC):
     """ """
 
     def __init__(
@@ -61,11 +62,11 @@ class Query(object):
         self._limit: LimitClause = limit
 
     @classmethod
-    def _builder(cls, engine: Optional[object] = None) -> object:
+    def _builder(cls, engine: Optional["Engine"] = None) -> "QueryBuilder":
         return QueryBuilder(engine)
 
     @classmethod
-    def from_builder(cls, builder: object):
+    def from_builder(cls, builder: "QueryBuilder") -> "Query":
         """Create Query object from QueryBuilder.
         Kind of reverse process"""
         if not IQueryBuilder(builder)._finalized:
@@ -82,31 +83,31 @@ class Query(object):
         )
         return query
 
-    def get_where(self):
+    def get_where(self) -> WhereClause:
         """ """
         return self._where
 
-    def get_from(self):
+    def get_from(self) -> FromClause:
         """ """
         return self._from
 
-    def get_select(self):
+    def get_select(self) -> SelectClause:
         """ """
         return self._select
 
-    def get_sort(self):
+    def get_sort(self) -> SortClause:
         """ """
         return self._sort
 
-    def get_limit(self):
+    def get_limit(self) -> LimitClause:
         """ """
         return self._limit
 
-    def clone(self):
+    def clone(self) -> "Query":
         """ """
         return self.__copy__()
 
-    def __copy__(self):
+    def __copy__(self) -> "Query":
         """ """
         newone = type(self).__new__(type(self))
         newone.__dict__.update(self.__dict__)
@@ -128,30 +129,30 @@ class Query(object):
 
 
 @implementer(IQueryBuilder)
-class QueryBuilder(object):
+class QueryBuilder(ABC):
     """ """
 
-    def __init__(self, engine=None):
+    def __init__(self, engine: Optional["Engine"] = None):
         """ """
-        self._engine = engine
-        self._finalized = False
+        self._engine: Optional["Engine"] = engine
+        self._finalized: bool = False
 
-        self._from = FromClause()
-        self._select = SelectClause()
-        self._where = WhereClause()
-        self._sort = SortClause()
-        self._limit = LimitClause()
+        self._from: FromClause = FromClause()
+        self._select: SelectClause = SelectClause()
+        self._where: WhereClause = WhereClause()
+        self._sort: SortClause = SortClause()
+        self._limit: LimitClause = LimitClause()
 
-    def bind(self, engine):
+    def bind(self, engine: "Engine"):
         """ """
         # might be clone
         self._engine = engine
 
-    def clone(self):
+    def clone(self) -> "QueryBuilder":
         """ """
         return self.__copy__()
 
-    def finalize(self, engine=None):
+    def finalize(self, engine: Optional["Engine"] = None):
         """ """
         self._pre_check()
 
@@ -160,9 +161,7 @@ class QueryBuilder(object):
 
         if self._engine is None:
             raise ConstraintNotSatisfied(
-                "Object from '{0!s}' must be binded with engine".format(
-                    self.__class__.__name__
-                )
+                f"Object from '{self.__class__.__name__}' must be bound with engine"
             )
         # xxx: do any validation?
         if len(self._select) == 0:
@@ -182,7 +181,7 @@ class QueryBuilder(object):
 
         self._finalized = True
 
-    def __copy__(self):
+    def __copy__(self) -> "QueryBuilder":
         """ """
         newone = type(self).__new__(type(self))
         newone.__dict__.update(self.__dict__)
@@ -199,17 +198,18 @@ class QueryBuilder(object):
         return newone
 
     @builder
-    def from_(self, resource_type, alias=None):
+    def from_(self, resource_type: Union[str, List[str]]):
         """ """
         required_not_finalized(self)
 
-        if len(self._from) > 0:
-            # info: we are allowing single resource only
-            raise ValidationError("from_ value already assigned!")
-
-        model = Model.create(resource_type)
-        alias = alias or model.resource_type
-        self._from.append((alias, model))
+        assert self._engine
+        if isinstance(resource_type, str):
+            model = Model.create(resource_type, fhir_release=self._engine.fhir_release)
+            self._from.append((resource_type, model))
+        else:
+            for r_type in resource_type:
+                model = Model.create(r_type, fhir_release=self._engine.fhir_release)
+                self._from.append((r_type, model))
 
     @builder
     def select(self, *args):
@@ -262,7 +262,7 @@ class QueryBuilder(object):
                     sort_path = sort_(sort_path)
             self._sort.append(sort_path)
 
-    def get_query(self):
+    def get_query(self) -> "Query":
         """ """
         required_finalized(self)
 
@@ -280,7 +280,12 @@ class QueryBuilder(object):
 
         return self.__fql__()
 
-    def __call__(self, unrestricted=False, engine=None, async_result=False):
+    def __call__(
+        self,
+        unrestricted: bool = False,
+        engine: Optional["Engine"] = None,
+        async_result: bool = False,
+    ) -> Union["QueryResult", "AsyncQueryResult"]:
         """ """
         if not self._finalized and (engine or self._engine):
             self.finalize(engine)
@@ -289,7 +294,8 @@ class QueryBuilder(object):
         result_factory = QueryResult
         if async_result is True:
             result_factory = AsyncQueryResult
-
+        if TYPE_CHECKING:
+            assert self._engine
         result = result_factory(
             query=query, engine=self._engine, unrestricted=unrestricted
         )
@@ -297,7 +303,8 @@ class QueryBuilder(object):
 
     def _pre_check(self):
         """ """
-        required_from_resource(self)
+        # TODO can we modify this check somehow?
+        # required_from_resource(self)
         required_not_finalized(self)
 
     def _validate(self):
@@ -306,18 +313,19 @@ class QueryBuilder(object):
         if any([el.star for el in self._select]) and len(self._select) > 1:
             raise ValidationError("select(*) cannot co-exists other select element!")
 
-    def _validate_root_path(self, path_string):
+    def _validate_root_path(self, path_string: str):
         """ """
-        match = False
-        for alias, model in self._from:
-            if path_string.split(".")[0] == alias:
-                match = True
-                break
-        if match is False:
+        root_path = path_string.split(".")[0]
+
+        if self._from:
+            match = any(alias == root_path for alias, _ in self._from)
+        else:
+            # FIXME: find a better way to validate that we're searching on all resources
+            match = root_path == "Resource"
+
+        if not match:
             raise ValidationError(
-                "Root path '{0!s}' must be matched with from models".format(
-                    path_string.split(".")[0]
-                )
+                f"Root path '{root_path}' must be matched with from models"
             )
 
     def _validate_term_path(self, term):
@@ -330,19 +338,18 @@ class QueryBuilder(object):
 
 
 @implementer(IQueryResult)
-class QueryResult(object):
+class QueryResult(ABC):
     """ """
 
-    def __init__(self, query: Query, engine, unrestricted=False):
+    def __init__(self, query: Query, engine: "Engine", unrestricted: bool = False):
         """ """
-        self._query = query
-        self._engine = engine
-        self._unrestricted = unrestricted
+        self._query: Query = query
+        self._engine: "Engine" = engine
+        self._unrestricted: bool = unrestricted
 
     def fetchall(self):
         """ """
-        result = self._engine.execute(self._query, self._unrestricted)
-        return result
+        return self._engine.execute(self._query, self._unrestricted)
 
     def single(self):
         """Will return the single item in the input if there is just one item.
@@ -395,10 +402,9 @@ class QueryResult(object):
         """Returns a collection with a single value which is the integer count of
         the number of items in the input collection.
         Returns 0 when the input collection is empty."""
-        result = self._engine.execute(
+        return self._engine.execute(
             self._query, self._unrestricted, EngineQueryType.COUNT
-        )
-        return result.header.total
+        ).header.total
 
     def empty(self):
         """Returns true if the input collection is empty ({ }) and false otherwise."""
@@ -466,7 +472,7 @@ class QueryResult(object):
         model_class = self._query.get_from()[0][1]
         for row in result.body:
             if self._query.get_select()[0].star:
-                yield model_class(row[0])
+                yield model_class(**row[0])
             else:
                 yield row
 
@@ -485,7 +491,7 @@ class AsyncQueryResult(QueryResult):
         model_class = self._query.get_from()[0][1]
         for row in result.body:
             if self._query.get_select()[0].star:
-                yield model_class(row[0])
+                yield model_class(**row[0])
             else:
                 yield row
 
@@ -509,10 +515,9 @@ class AsyncQueryResult(QueryResult):
 
     async def count(self):
         """ """
-        result = await self._engine.execute(
+        return await self._engine.execute(
             self._query, self._unrestricted, EngineQueryType.COUNT
-        )
-        return result.header.total
+        ).header.total
 
     async def empty(self):
         """Returns true if the input collection is empty ({ }) and false otherwise."""
@@ -523,9 +528,9 @@ class AsyncQueryResult(QueryResult):
         return self.count()
 
 
-def Q_(resource=None, engine=None):
+def Q_(resource: Optional[Union[str, List[str]]] = None, engine=None):
     """ """
     builder = Query._builder(engine)
-    if resource is not None:
+    if resource:
         builder = builder.from_(resource)
     return builder
